<?php

/**
 * Back-End of Utf8_XH.
 *
 * Copyright (c) 2012 Christoph M. Becker (see license.txt)
 */

// utf-8-marker: äöüß


if (!defined('CMSIMPLE_XH_VERSION')) {
    header('HTTP/1.0 403 Forbidden');
    exit;
}


define('UTF8_VERSION', '1beta1');


/**
 * Returns the plugin version information view.
 *
 * @return string  The (X)HTML.
 */
function utf8_version() {
    return '<h1><a href="http://3-magi.net/?CMSimple_XH/Utf8_XH">Utf8_XH</a></h1>'."\n"
	    .'<p>Version: '.UTF8_VERSION.'</p>'."\n"
	    .'<p>Copyright &copy; 2012 <a href="http://3-magi.net">Christoph M. Becker</a></p>'."\n"
	    .'<p style="text-align: justify">This program is free software: you can redistribute it and/or modify'
	    .' it under the terms of the GNU General Public License as published by'
	    .' the Free Software Foundation, either version 3 of the License, or'
	    .' (at your option) any later version.</p>'."\n"
	    .'<p style="text-align: justify">This program is distributed in the hope that it will be useful,'
	    .' but WITHOUT ANY WARRANTY; without even the implied warranty of'
	    .' MERCHAN&shy;TABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the'
	    .' GNU General Public License for more details.</p>'."\n"
	    .'<p style="text-align: justify">You should have received a copy of the GNU General Public License'
	    .' along with this program.  If not, see'
	    .' <a href="http://www.gnu.org/licenses/">http://www.gnu.org/licenses/</a>.</p>'."\n";
}


/**
 * Returns the requirements information view.
 *
 * @return string  The (X)HTML.
 */
function utf8_system_check() { // RELEASE-TODO
    global $pth, $tx, $plugin_tx;

    define('UTF8_PHP_VERSION', '4.3.0');
    $ptx = $plugin_tx['utf8'];
    $imgdir = $pth['folder']['plugins'].'utf8/images/';
    $ok = tag('img src="'.$imgdir.'ok.png" alt="ok"');
    $warn = tag('img src="'.$imgdir.'warn.png" alt="warning"');
    $fail = tag('img src="'.$imgdir.'fail.png" alt="failure"');
    $htm = tag('hr').'<h4>'.$ptx['syscheck_title'].'</h4>'
	    .(version_compare(PHP_VERSION, UTF8_PHP_VERSION) >= 0 ? $ok : $fail)
	    .'&nbsp;&nbsp;'.sprintf($ptx['syscheck_phpversion'], UTF8_PHP_VERSION)
	    .tag('br').tag('br')."\n";
    foreach (array('mbstring', 'pcre') as $ext) {
	$htm .= (extension_loaded($ext) ? $ok : $fail)
		.'&nbsp;&nbsp;'.sprintf($ptx['syscheck_extension'], $ext).tag('br')."\n";
    }
    $htm .= tag('br').(strtoupper($tx['meta']['codepage']) == 'UTF-8' ? $ok : $warn)
	    .'&nbsp;&nbsp;'.$ptx['syscheck_encoding'].tag('br')."\n";
    $htm .= (!get_magic_quotes_runtime() ? $ok : $fail)
	    .'&nbsp;&nbsp;'.$ptx['syscheck_magic_quotes'].tag('br')."\n";
    $htm .= (file_exists($pth['folder']['plugins'].'jquery/jquery.inc.php') ? $ok : $fail)
	    .'&nbsp;&nbsp;'.$ptx['syscheck_jquery'].tag('br').tag('br')."\n";
    foreach (array('config/', 'css/', 'languages/') as $folder) {
	$folders[] = $pth['folder']['plugins'].'utf8/'.$folder;
    }
    foreach ($folders as $folder) {
	$htm .= (is_writable($folder) ? $ok : $warn)
		.'&nbsp;&nbsp;'.sprintf($ptx['syscheck_writable'], $folder).tag('br')."\n";
    }
    return $htm;
}


/**
 * Writes the JS to <head>.
 *
 * @global string $hjs
 * @return void
 */
function utf8_js() {
    global $pth, $hjs, $plugin_tx;

    include_once $pth['folder']['plugins'].'jquery/jquery.inc.php';
    include_jQuery();
    $hjs .= <<<SCRIPT
<script type="text/javascript">
/* <![CDATA[ */
jQuery(function() {
    jQuery('#utf8_admin').submit(function() {
	return confirm('{$plugin_tx['utf8']['confirm_conversion']}');
    })
    jQuery('span.utf8_toggler').click(function() {
	jQuery(this).parent('li').children('ul').toggle();
    })
})
/* ]]> */
</script>

SCRIPT;
}


/**
 * Returns the view of the encoding selectbox.
 *
 * @return string  The (X)HTML.
 */
function utf8_encoding_selectbox() {
    global $plugin_tx;

    $o = '<select name="utf8_from">';
    $encodings[] = $plugin_tx['utf8']['label_convert_from'];
    for ($i = 1; $i <= 15; $i++) {$encodings[] = 'ISO-8859-'.$i;}
    foreach (array('CP1251', 'CP1252', 'KOI8-R') as $encoding) {
	$encodings[] = $encoding;
    }
    foreach ($encodings as $encoding) {
	$o .= '<option>'.$encoding.'</option>';
    }
    $o .= '</select>';
    return $o;
}


/**
 * Returns the view of the symbol.
 *
 * @param string $name  The name of the symbol.
 * @return string  The (X)HTML.
 */
function utf8_symbol($name) {
    global $pth;

    return tag('img src="'.$pth['folder']['plugins'].'utf8/images/'.$name.'.png"');
}


/**
 * Returns the state of the file.
 *
 * @param string $fn  The file name.
 * @return string  The state (ok, warn, fail, unknown).
 */
function utf8_state($fn) {
    if (!is_readable($fn)) {return 'unknown';}
    $txt = file_get_contents($fn);
    if (substr($txt, 0 ,3) == pack('CCC', 0xef, 0xbb, 0xbf)) {
	return 'fail';
    }
    return preg_match('/^.*$/su', $txt) ? 'ok' : 'fail';
}


/**
 * Returns the tree view of a directory.
 *
 * @param string $dir
 * @return string  The (X)HTML.
 */
function utf8_traverse_files($dir) {
    global $plugin_cf;

    set_time_limit(30);
    $exts = explode(',', $plugin_cf['utf8']['extensions']);
    if ($dh = opendir($dir)) {
	$o = '<ul>'."\n";
	while (($file = readdir($dh)) !== FALSE) {
	    if ($file != '.' && $file != '..') {
		if (is_dir($dir.$file)) {
		    $sub = utf8_traverse_files($dir.$file.'/');
		    $o .= '<li>'.utf8_symbol($sub !== FALSE ? 'folder' : 'unknown')
			    .' <span class="utf8_toggler">'.$file.'</span>'
			    .$sub.'</li>'."\n";
		} else {
		    if (in_array(pathinfo($file, PATHINFO_EXTENSION), $exts)) {
			$state = utf8_state($dir.$file);
			$symb = utf8_symbol($state);
			$cb = $state == 'fail' || $state == 'warn'
				? tag('input type="checkbox" name="utf8_files[]" value="'.$dir.$file.'"').' '
				: '';
			$o .= '<li>'.$symb.' '.$cb .$file.'</li>'."\n";
		    }
		}
	    }
	}
	closedir($dh);
	$o .= '</ul>'."\n";
    } else {
	$o = FALSE;
    }
    return $o;
}


/**
 * Returns the tree view of all files.
 *
 * @return string  The (X)HTML.
 */
function utf8_admin_main() {
    global $pth, $sn, $plugin_tx;

    utf8_js();
    $url = $sn.'?&amp;utf8&amp;admin=plugin_main&amp;action=convert';
    return '<form id="utf8_admin" action="'.$url.'" method="POST">'."\n"
	    .utf8_traverse_files($pth['folder']['base'])
	    .utf8_encoding_selectbox()."\n"
	    .tag('input type="submit" class="submit" value="'.$plugin_tx['utf8']['label_convert'].'"')."\n"
	    .'<form>'."\n";
}


/**
 * Converts the posted files to UTF-8.
 *
 * @return string  The admin main view.
 */
function utf8_convert() {
    global $plugin_tx;

    $from = stsl($_POST['utf8_from']);
    if ($from != $plugin_tx['utf8']['label_convert_from']) {
	foreach ($_POST['utf8_files'] as $file) {
	    set_time_limit(30);
	    $file = stsl($file);
	    if (($txt = file_get_contents($file)) !== FALSE) {
		if (substr($txt, 0 ,3) == pack('CCC', 0xef, 0xbb, 0xbf)) {
		    $txt = substr($txt, 3); // remove BOM
		} else {
		    $txt = mb_convert_encoding($txt, 'UTF-8', $from);
		}
		if (($fp = fopen($file, 'w')) !== FALSE) {
		    fwrite($fp, $txt);
		    fclose($fp);
		}
	    }
	}
    }
    return utf8_admin_main();
}


/**
 * Handle the plugin administration.
 */
initvar('utf8');
if (!empty($utf8)) {
    initvar('admin');
    initvar('action');

    $o .= print_plugin_admin('on');

    switch($admin) {
	case '':
	    $o .= utf8_version().utf8_system_check();
	    break;
	case 'plugin_main':
	    switch ($action) {
		case 'plugin_text':
		    $o .= utf8_admin_main();
		    break;
		case 'convert':
		    $o .= utf8_convert();
		    break;
	    }
	    break;
	default:
	    $o .= plugin_admin_common($action, $admin, $plugin);
    }
}

?>
